﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                            BASS Audio class                              |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 10-17-2006 (MM-DD-YYYY)            |
//|                        Last revised : 10-17-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Runtime.InteropServices;

namespace Audio
{
    class BASS
    {
        private const string BASSDLL = "bass.dll";

        [DllImport(BASSDLL)]
        public static extern bool BASS_Init(int device, uint freq, uint flag, IntPtr hParent, uint GUID);
        [DllImport(BASSDLL)]
        public static extern bool BASS_Free();
        [DllImport(BASSDLL)]
        public static extern bool BASS_StreamFree(uint Channel);
        [DllImport(BASSDLL)]
        public static extern uint BASS_StreamCreateFile(bool mem, string file, uint offset, uint length, uint flags);
        [DllImport(BASSDLL)]
        public static extern bool BASS_ChannelPlay(uint Channel, bool restart);
        [DllImport(BASSDLL)]
        public static extern bool BASS_ChannelStop(uint Channel);
        [DllImport(BASSDLL)]
        public static extern uint BASS_ChannelGetLevel(uint Channel);

        private static uint level = 0;
        public static uint Level
        {
            get { return level; }
            set { level = value; }
        }

        private static string filetoplay = "";
        public static string FileToPlay
        {
            get { return filetoplay; }
            set { filetoplay = value; }
        }

        private static uint channel = 0;
        public static uint Channel
        {
            get { return channel; }
            set { channel = value; }
        }

        private static bool bassinit = false;
        public static bool Init(IntPtr hParent)
        {
            if (!bassinit)
            {
                bassinit = BASS_Init(1, 44100, 0, hParent, 0);
            }
            return bassinit;
        }

        public static bool Free()
        {
            bool nRet = false;
            if (bassinit) nRet = BASS_Free(); bassinit = false;
            return nRet;
        }

        public static void StreamFree()
        {
            if (Channel != 0) BASS_StreamFree(Channel); Channel = 0; Level = 0;
        }

        public static void PlayStream(IntPtr hParent)
        {
            if (Init(hParent))
            {
                StreamFree();
                //	sFileToPlay is string = ExePath + "plasticz_psynus.mp3"
                if (File.Exists(FileToPlay))
                {
                    Channel = BASS_StreamCreateFile(false, FileToPlay, 0, 0, 0);
                    BASS_ChannelPlay(Channel, false);
                }
            }
        }

        //public static bool ChannelStop(uint UseChannel)
        //{
        //    bool nRet = false;
        //    if (UseChannel != 0) nRet = BASS_ChannelStop(UseChannel);
        //    return nRet;
        //}

        public static uint ChannelGetLevel(uint UseChannel)
        {
            if (UseChannel != 0)
            {
                uint GetLevel = BASS_ChannelGetLevel(UseChannel);
                // Check for stealth mode (no more sound).
                if (GetLevel == 4294967295) GetLevel = 0;
                Level = GetLevel;
            }
            else
            {
                Level = 0;
            }
            return Level;
        }

    }
}
