﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                                BlackBox                                  |
//|                             Version  1.00                                |
//|                                                                          |
//|         This project mimics the VISTA AERO GLASS style under XP.         |
//|              It combines OpenGL and BASS audio altogether.               |
//|                                                                          |
//|         Many features inherit from the Win32 WinLIFT SkinEngine.         |
//|                     (Skin design Patrice Terrier)                        |
//|                                                                          |
//|                    BASS.DLL (c) copyright Ian Luck                       |
//| If you are not charging for your software and it has no other commercial |
//| purpose (advertising/promotion), then you can use BASS in it for free.   |
//|              You can find the latest BASS.DLL version there:             |
//|                            www.un4seen.com                               |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 10-09-2006 (MM-DD-YYYY)            |
//|                        Last revised : 10-18-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
//using System.Collections.Generic;
//using System.ComponentModel;
//using System.Data;
//using System.Drawing.Drawing2D;
//using System.Drawing.Imaging;
//using System.Runtime.InteropServices;
//using System.IO;

using Win32;
using OpenGL;
using Audio;
using SkinEngine;

//using System.Threading;
//using System.Runtime;
//using System.Diagnostics;

namespace BlackBox
{
    public partial class MAIN_Form : Form
    {
        const int TIMER_MAX = 1;                               // To perform smooth animation

        static IntPtr hGLcontrol = IntPtr.Zero;         // Default to Zero

        static IntPtr hFORM_Main = IntPtr.Zero;         // Default to Zero
        static IntPtr hBACK_BtnSys = IntPtr.Zero;       // Default to Zero
        static IntPtr hBackGround = IntPtr.Zero;        // Default to Zero

        public MAIN_Form()
        {
            hFORM_Main = this.Handle;

            InitializeComponent();

            hGLcontrol = OPENGL_Container.Handle;
        }

        private void SETUP_parameters()
        {
            // Setup default Skin colors
            SK.ColorLayerBackground = Color.FromArgb(255, 64, 64, 64);
            SK.ColorCaptionEnabled = Color.FromArgb(255, 220, 220, 220);
            SK.ColorCaptionDisabled = Color.FromArgb(255, 92, 92, 92);

            // Skin opacity in the range 0-255
            SK.Alpha = 204; // 80 % default transparency level

            // Skin opacity of the hidden background in the range 0-254
            SK.OpacityLayerBackground = 248;

            //STATUS_Label.ForeColor = Skin.ColorCaptionEnabled;

            // Caption text + color
            if (this.Enabled)
            { CAPTION_Label.ForeColor = SK.ColorCaptionEnabled; }
            else
            { CAPTION_Label.ForeColor = SK.ColorCaptionDisabled; }
            CAPTION_Label.TextAlign = ContentAlignment.MiddleLeft;
            CAPTION_Label.Text = "BLACK Box II";//this.Text;

            // OpenGL
            glInitialize();
            StartAnimation();
            // Try playing with these parameters
            Pulsating = true; // Rem this out to disable pulsating effect
            Velocity = true;  // Rem this out to disable velocity effect

            // BASS.dll
            BASS.Init(hFORM_Main);
            // Default audio file (Edit to use yours)
            BASS.FileToPlay = @"BlackBox.mp3";

            // Drag & drop 
            // It accepts only mp3 files dragged from Windows Explorer
            Api.DragAcceptFiles(hFORM_Main, true);

        }

        private void FORM_Load(object sender, EventArgs e)
        {
            // We need this to retrieve bitmap from resource
            SK.UseNameSpace = this.GetType().Namespace;

            // Use bitmap resources to setup public value for:
            // FixCaptionLeft, FixCaptionHeight, FixCaptionRight, FixBottomHeight.
            Bitmap bmp;
            bmp = Properties.Resources.TopLeft;
            SK.FixCaptionLeft = bmp.Width; SK.FixCaptionHeight = bmp.Height;
            bmp = Properties.Resources.BottomRight;
            SK.FixCaptionRight = bmp.Width; SK.FixBottomHeight = bmp.Height;
            SK.DockMin = SK.FixCaptionHeight + SK.FixBottomHeight; // Docking

            // Setup the minimum size of the form
            this.MinimumSize = new System.Drawing.Size(SK.FixCaptionLeft + SK.FixCaptionRight, SK.FixCaptionHeight + SK.FixBottomHeight);

            // Create background and setup inital region
            this.BackgroundImage = (Bitmap)SK.DrawBackGround(this.Width, this.Height);

            Bitmap bmpFrmBack = (Bitmap)this.BackgroundImage;
            SK.CreateFormRegion(this, bmpFrmBack);
            //RECT rw = new RECT();
            Api.RECT rw = new Api.RECT();
            Api.GetWindowRect(hFORM_Main, ref rw);
            SK.LastRgnWidth = rw.right - rw.left; SK.LastRgnHeight = rw.bottom - rw.top;

            //IsResizable = false;

            // Check first if we can use layered window.
            byte Alpha = 0;
            if (Api.GetOsVersion() > 499)
            {
                Alpha = 254; this.Opacity = (double)Alpha / 255;
            }
            else
            {
                Alpha = 255; this.Opacity = (double)Alpha / 255;
            }
            // Support for layered window is a prerequisite to create aero glass effect
            if (Alpha < 255)
            {
                // Setup the parameters
                SETUP_parameters();

                //Create the skin background
                Form BackGround = new Form();
                BackGround.ShowInTaskbar = false; // WS_EX_TOOLWINDOW
                BackGround.Visible = true;
                BackGround.FormBorderStyle = 0;
                BackGround.BackColor = SK.ColorLayerBackground;
                BackGround.Opacity = 0.999; // Don't change this !
                hBackGround = BackGround.Handle;
                Api.SetLayeredWindowAttributes(hBackGround, 0, SK.OpacityLayerBackground, 0x00000002);

                ResizeMoveHiddenWindows();

                // REM this out to disable the drop shadow effect.
                //Api.UseDropShadow(hFORM_Main);

                Api.SetLayeredWindowAttributes(hFORM_Main, 0, SK.Alpha, 0x00000002);

            }
        }

        private void FORM_Close(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void ResizeMoveHiddenWindows()
        {
            if (Api.IsWindow(hBackGround))
            {
                if (SK.Alpha >= 254)
                {
                    Api.RECT r = new Api.RECT();
                    Api.GetWindowRect(hFORM_Main, ref r);
                    Api.MoveWindow(hBackGround, -9990, -9990, r.right - r.left, r.bottom - r.top, false);
                    return;
                }

                //Create hidden region
                IntPtr hRgn = Api.CreateRectRgn(0, 0, 0, 0);
                if (hRgn != IntPtr.Zero)
                {
                    if (Api.GetWindowRgn(hFORM_Main, hRgn))
                    {
                        IntPtr hRgnTemp = Api.CreateRectRgn(0, 0, CLIENT_Panel.Width, CLIENT_Panel.Height);
                        Api.OffsetRgn(hRgnTemp, CLIENT_Panel.Location.X, CLIENT_Panel.Location.Y);
                        Api.CombineRgn(hRgn, hRgn, hRgnTemp, Api.RGN_AND);
                        Api.DeleteObject(hRgnTemp);

                        hRgnTemp = Api.CreateRectRgn(0, 0, BTN_Iconize.Width + BTN_Maximize.Width + BTN_Close.Width, BTN_Iconize.Height);
                        Api.OffsetRgn(hRgnTemp, BTN_Iconize.Location.X, 0);
                        Api.CombineRgn(hRgn, hRgn, hRgnTemp, Api.RGN_OR);
                        Api.DeleteObject(hRgnTemp);

                        if (Api.IsWindow(BTN_DockIn.Handle))
                        {
                            hRgnTemp = Api.CreateRectRgn(0, 0, BTN_DockIn.Width - 3, BTN_DockIn.Height - 2);
                            Api.OffsetRgn(hRgnTemp, BTN_DockIn.Location.X + 1, 0);
                            Api.CombineRgn(hRgn, hRgn, hRgnTemp, Api.RGN_OR);
                            Api.DeleteObject(hRgnTemp);
                        }

                        if (Api.IsWindow(BTN_About.Handle))
                        {
                            hRgnTemp = Api.CreateRectRgn(0, 0, 0, 0);
                            if (Api.GetWindowRgn(BTN_About.Handle, hRgnTemp))
                            {
                                Api.OffsetRgn(hRgnTemp, BTN_About.Location.X, BTN_About.Location.Y);
                                Api.CombineRgn(hRgn, hRgn, hRgnTemp, Api.RGN_OR);
                            }
                            Api.DeleteObject(hRgnTemp);
                        }

                        // Set the new region to the form.
                        Api.SetWindowRgn(hBackGround, hRgn, false);
                    }
                }

                Api.RECT rw = new Api.RECT();
                uint swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING | Api.SWP_NOOWNERZORDER | Api.SWP_NOACTIVATE;
                Api.GetWindowRect(hFORM_Main, ref rw);
                // Must use MoveWindow to restore location after Iconize/Restore in case of layered window.
                Api.MoveWindow(hBackGround, rw.left, rw.top, rw.right - rw.left, rw.bottom - rw.top, false);
                Api.SetWindowPos(hBackGround, Api.GetWindow(hFORM_Main, 2), 0, 0, 0, 0, swpStyle);

            }
        }

        private void FORM_Move(object sender, EventArgs e)
        {
            ResizeMoveHiddenWindows();
        }

        private void FOCUS_Activated(object sender, EventArgs e)
        {
            if (Api.IsWindow(hBackGround))
            {
                uint swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING | Api.SWP_NOOWNERZORDER | Api.SWP_NOACTIVATE | Api.SWP_NOREDRAW;
                Api.SetWindowPos(hBackGround, IntPtr.Zero, 0, 0, 0, 0, swpStyle);
                swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING;
                Api.SetWindowPos(hFORM_Main, IntPtr.Zero, 0, 0, 0, 0, swpStyle);
            }
            if (LED_Off.Visible)
            {
                CAPTION_Label.ForeColor = SK.ColorCaptionEnabled;
                LED_On.Visible = true;
                LED_Off.Visible = false;
            }
        }

        private void FOCUS_Deactivated(object sender, EventArgs e)
        {
            CAPTION_Label.Select();
            CAPTION_Label.ForeColor = SK.ColorCaptionDisabled;
            LED_On.Visible = false;
            LED_Off.Visible = true;
        }

        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {
                case Api.WM_SIZE:

                    //Start flicker free section
                    //StopAnimation();
                    Api.ControlDrawingDisable(CLIENT_Panel.Handle);

                    SK.BuildRegion(m.HWnd);
                    ResizeMoveHiddenWindows();
                    ResizeGLcontrol();

                    //End flicker free section
                    Api.ControlDrawingEnable(CLIENT_Panel.Handle);
                    //StartAnimation();

                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_DROPFILES: // Drag & drop
                    uint hDrop = (uint)m.WParam;
                    int FilesDropped = Api.DragQueryFile(hDrop, -1, null, 0);
                    string MediaFile = "";
                    if (FilesDropped != 0)
                    {
                        StringBuilder sFileName = new StringBuilder(Api.MAX_PATH);

                        for (int i = 0; i < FilesDropped; i++)
                        {
                            Api.DragQueryFile(hDrop, i, sFileName, Api.MAX_PATH);
                            MediaFile = sFileName.ToString().ToLower(); break;
                        }
                    }
                    Api.DragFinish(hDrop);
                    if (MediaFile.Length != 0)
                    {
                        if (MediaFile.EndsWith(".mp3"))
                        {
                            BASS.FileToPlay = MediaFile;
                            BASS.PlayStream(hFORM_Main);
                        }
                    }
                    break;

                case Api.WM_TIMER:
                    int wParam = (int)m.WParam;
                    if (wParam == TIMER_MAX + 1)
                    {
                        BASS.ChannelGetLevel(BASS.Channel);
                    }
                    else
                    {
                        RenderOpenGL();
                    }
                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_ERASEBKGND:
                    m.Result = (IntPtr)1;
                    return;

                case Api.WM_NCLBUTTONDBLCLK:
                    // We handle this ourself to avoid havoc when using restore
                    // when using a form without caption.
                    if (SK.IsMaximized)
                    {
                        Api.ButtonClick(BTN_Restore.Handle);
                    }
                    else
                    {
                        Api.ButtonClick(BTN_Maximize.Handle);
                    }
                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_LBUTTONDBLCLK:
                    if (SK.IsMaximized)
                    { // We simulate double click on caption to restore the window size
                        if (Api.HiWrd((uint)m.LParam) < SK.FixCaptionHeight) Api.ButtonClick(BTN_Restore.Handle);
                    }
                    break;

                case Api.WM_DESTROY:
                    DisposeGL(); // Free up OpenGL resources
                    BASS.Free(); // Free up BASS resources
                    break;

                case Api.WM_ACTIVATEAPP: // Keep hidden background behind the form
                    if (Api.IsWindow(hBackGround))
                    {
                        if ((uint)m.WParam == 0)
                        {
                            uint swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING | Api.SWP_NOOWNERZORDER | Api.SWP_NOACTIVATE | Api.SWP_NOREDRAW;
                            Api.SetWindowPos(hBackGround, Api.GetWindow(hFORM_Main, 2), 0, 0, 0, 0, swpStyle);
                        }
                    }
                    break;
            }

            base.WndProc(ref m);

            // This is the place where the skin NC area is handled.
            if (m.Msg == Api.WM_NCHITTEST)
            {
                if (SK.IsMaximized) return;

                if (m.HWnd == CAPTION_Label.Handle)
                {
                    m.Result = (IntPtr)Api.HTCAPTION;
                    return;
                }

                if (m.Result == (IntPtr)Api.HTCLIENT)
                {
                    int HITTEST = Api.HTCAPTION;
                    if (SK.IsResizable) // Check if the form is resizable
                    {
                        Point p = this.PointToClient(new Point(Api.LoWrd((uint)m.LParam), Api.HiWrd((uint)m.LParam)));
                        int xF = this.Width, yF = this.Height;
                        int xSide = 0;
                        int Border = System.Windows.Forms.SystemInformation.Border3DSize.Width;
                        if ((p.X >= xF - SK.FixBottomHeight) && ((p.Y >= yF - SK.FixBottomHeight)))
                        {
                            HITTEST = Api.HTBOTTOMRIGHT;
                        }
                        else
                        {
                            // Left side
                            if (p.X <= 8)
                            {
                                if (p.X <= Border) HITTEST = Api.HTLEFT;
                                xSide = 1;
                            }
                            // Right side
                            if (p.X >= xF - 8)
                            {
                                if (p.X >= xF - Border) HITTEST = Api.HTRIGHT;
                                xSide = 2;
                            }
                            // Top side
                            if (p.Y <= Border)
                            {
                                HITTEST = Api.HTTOP;
                                if (xSide == 1)
                                {
                                    HITTEST = Api.HTTOPLEFT;
                                }
                                else if (xSide == 2)
                                {
                                    HITTEST = Api.HTTOPRIGHT;
                                }
                            }
                            // Bottom side
                            if (p.Y >= yF - Border)
                            {
                                if (xSide == 1)
                                {
                                    HITTEST = Api.HTBOTTOMLEFT;
                                }
                                else
                                {
                                    HITTEST = Api.HTBOTTOM;
                                }
                            }
                        }
                    }

                    m.Result = (IntPtr)HITTEST;
                }
            }
        }

        private void BTN_Close_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void BTN_Iconize_Click(object sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Minimized;
        }

        private int WasX, WasY, WasWidth, WasHeight;
        private void BTN_Restore_Click(object sender, EventArgs e)
        {
            if (SK.IsDocked)
            {
                BTN_DockOut.Visible = true;
                CLIENT_Panel.Visible = false;
            }
            else
            {
                BTN_DockIn.Visible = true;
                CLIENT_Panel.Visible = true;
            }
            BTN_Maximize.Visible = true;
            BTN_Restore.Visible = false;

            Api.SetLayeredWindowAttributes(hFORM_Main, 0, SK.Alpha, 0x00000002);
            Api.MoveWindow(hFORM_Main, WasX, WasY, WasWidth, WasHeight, false);

            SK.IsMaximized = false;

        }

        private void BTN_Maximize_Click(object sender, EventArgs e)
        {
            WasX = this.Location.X;
            WasY = this.Location.Y;
            WasWidth = this.Width;
            WasHeight = this.Height;

            Api.ControlDrawingDisable(this.Handle);

            Api.MoveWindow(hFORM_Main, 0, 0, SystemInformation.WorkingArea.Width, SystemInformation.WorkingArea.Height, false);

            BTN_DockIn.Visible = false;
            BTN_DockOut.Visible = false;
            BTN_Restore.Visible = true;
            BTN_Maximize.Visible = false;

            CLIENT_Panel.Visible = true;

            // Turn it to opaque mode
            Api.SetLayeredWindowAttributes(hFORM_Main, 0, 255, 0x00000002);

            SK.IsMaximized = true;

            Api.ControlDrawingEnable(this.Handle);
        }

        private void BTN_DockOut_Click(object sender, EventArgs e)
        {
            BTN_DockIn.Visible = true;
            BTN_DockOut.Visible = false;
            // Restore original window size
            this.Height = SK.DockHeightMax;
            this.Size = new Size(this.Width, SK.DockHeight);
            CLIENT_Panel.Visible = true;
            SK.IsDocked = false;
        }

        private void BTN_DockIn_Click(object sender, EventArgs e)
        {
            BTN_DockOut.Visible = true;
            BTN_DockIn.Visible = false;
            // Save current height of the window
            SK.DockHeight = this.Height;
            SK.DockHeightMax = this.MaximumSize.Height;
            this.Size = new Size(this.Width, SK.DockMin);
            CLIENT_Panel.Visible = false;
            SK.IsDocked = true;
        }

        private void BTN_About_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Author Patrice Terrier \n   copyright (c) 2006\nwww.zapsolution.com", "About...");
        }

        private void BTN_Url_Click(object sender, EventArgs e)
        {
            // Display the appropriate link based on the value of the 
            // LinkData property of the Link object.
            string target = "www.zapsolution.com";

            // If the value looks like a URL, navigate to it.
            // Otherwise, display it in a message box.
            if (null != target)
            {
                System.Diagnostics.Process.Start(target);
            }
        }

        private void BTN_Play_Click(object sender, EventArgs e)
        {
            BASS.PlayStream(hFORM_Main);
        }

        private void BTN_Stop_Click(object sender, EventArgs e)
        {
            BASS.StreamFree();
        }

        // ********************************************************
        // ********************************************************
        // ********************************************************
        static public int glDC = 0;              // Display device context
        static public int glRC = 0;              // OpenGL rendering context
        static public double dXangle = 0.0;      // OpenGL X Angle
        static public double dYangle = 0.0;      // OpenGL Y Angle
        static public double dZangle = 0.0;      // OpenGL Z Angle

        public static void StartAnimation()
        {
            // Use API WM_TIMER to avoid aggressive garbage collection 
            // to occur before Main ends.
            for (int T = 0; T < TIMER_MAX; T++)
            {
                Api.SetTimer(hFORM_Main, T + 1, 0, null);
            }
            Api.SetTimer(hFORM_Main, TIMER_MAX + 1, 33, null);
        }

        static int[] ColorArray = new int[34];
        public static void InitColorArray()
        {
            ColorArray[0] = Api.RGB(0, 0, 0);
            ColorArray[1] = Api.RGB(16, 16, 16);
            ColorArray[2] = Api.RGB(0, 44, 233);
            ColorArray[3] = Api.RGB(0, 67, 210);
            ColorArray[4] = Api.RGB(0, 89, 187);
            ColorArray[5] = Api.RGB(0, 112, 164);
            ColorArray[6] = Api.RGB(0, 135, 142);
            ColorArray[7] = Api.RGB(0, 159, 117);
            ColorArray[8] = Api.RGB(0, 183, 88);
            ColorArray[9] = Api.RGB(0, 207, 58);
            ColorArray[10] = Api.RGB(0, 231, 29);
            ColorArray[11] = Api.RGB(26, 234, 26);
            ColorArray[12] = Api.RGB(52, 237, 23);
            ColorArray[13] = Api.RGB(79, 240, 20);
            ColorArray[14] = Api.RGB(105, 243, 17);
            ColorArray[15] = Api.RGB(126, 245, 14);
            ColorArray[16] = Api.RGB(147, 248, 11);
            ColorArray[17] = Api.RGB(168, 250, 8);
            ColorArray[18] = Api.RGB(189, 253, 5);
            ColorArray[19] = Api.RGB(210, 255, 2);
            ColorArray[20] = Api.RGB(233, 255, 0);
            ColorArray[21] = Api.RGB(255, 255, 0);
            ColorArray[22] = Api.RGB(255, 251, 0);
            ColorArray[23] = Api.RGB(255, 235, 0);
            ColorArray[24] = Api.RGB(255, 215, 0);
            ColorArray[25] = Api.RGB(255, 196, 0);
            ColorArray[26] = Api.RGB(255, 176, 0);
            ColorArray[27] = Api.RGB(255, 156, 0);
            ColorArray[28] = Api.RGB(253, 137, 0);
            ColorArray[29] = Api.RGB(255, 117, 0);
            ColorArray[30] = Api.RGB(255, 97, 0);
            ColorArray[31] = Api.RGB(255, 78, 0);
            ColorArray[32] = Api.RGB(255, 58, 0);
            ColorArray[33] = Api.RGB(255, 0, 0);
        }

        private static bool pulsating = false;
        public static bool Pulsating
        {
            get { return pulsating; }
            set { pulsating = value; }
        }

        private static bool velocity = false;
        public static bool Velocity
        {
            get { return velocity; }
            set { velocity = value; }
        }

//        public static void glInitialize()
        private void glInitialize()
        {
            glDC = Api.GetDC(hGLcontrol);

            // SwapBuffers here, or you won't see anything !
            // ********************************************
            GL.wglSwapBuffers(glDC);
            // ********************************************

            if (GL.SetupPixelFormat(glDC))
            {
                glRC = GL.wglCreateContext(glDC);
                if (glRC != 0)
                {
                    if (GL.wglMakeCurrent(glDC, glRC))
                    {
                        GL.glEnable(GL.GL_DEPTH_TEST);
                        GL.glShadeModel(GL.GL_SMOOTH);

                        InitColorArray();

                        ResizeGLcontrol();

                        GL.glLoadIdentity();
                    }
                }
            }
            Api.ReleaseDC(hGLcontrol, glDC); glDC = 0;
        }

        public static void ResizeGLcontrol()
        {
            Api.RECT rc = new Api.RECT();
            Api.GetClientRect(hGLcontrol, ref rc);
            GL.glViewport(0, 0, rc.right, rc.bottom);

            GL.glMatrixMode(GL.GL_PROJECTION);
            GL.glLoadIdentity();
            GL.gluPerspective(35.0, GL.Fovy, 0.1, 150.0);
            GL.glMatrixMode(GL.GL_MODELVIEW);

            RenderOpenGL(); // Update OpenGL display
        }

        // Free up resources
        public static void DisposeGL()
        {
            if (glRC != 0)
            {
                GL.wglMakeCurrent(0, 0);
                GL.wglDeleteContext(glRC); glRC = 0;
            }
            for (int T = 0; T < TIMER_MAX; T++)
            {
                Api.KillTimer(hFORM_Main, T + 1);
            }
            Api.KillTimer(hFORM_Main, TIMER_MAX + 1);
        }

        private static int SolvePeak(int nValue, int nTotal)
        {
            int nRet = 0;
            if (nTotal != 0) nRet = (int)(nValue * 100) / nTotal;
            return nRet;
        }

        private static int LevelColr(int nLevel)
        {
            nLevel++; if (nLevel > 33) nLevel = 33;
            int Colr = 0; if (nLevel > 0) Colr = ColorArray[nLevel];
            return Colr;
        }

        public static void RenderOpenGL()
        {
            int glDC = Api.GetDC(hGLcontrol);
            if (glRC != 0)
            {
                uint nLevel = BASS.Level;// BASS.ChannelGetLevel(BASS.Channel); //gnLevel;
                int nLValue = (int)(SolvePeak(Api.LoWrd(nLevel), 128) / 700);
                int nRValue = (int)(SolvePeak(Api.HiWrd(nLevel), 128) / 768);

                dXangle += 0.3; // dXangle
                dYangle += 0.5; // dYangle
                dZangle += 0.7; // dZangle

                double pulse = (double)(Math.Max(nLValue, nRValue) / 7);
                // Pulsating effect section
                if (Pulsating)
                {
                    GL.glMatrixMode(GL.GL_PROJECTION);
                    GL.glLoadIdentity();
                    GL.gluPerspective(35.0 - pulse, GL.Fovy, 0.1, 150.0);
                    GL.glMatrixMode(GL.GL_MODELVIEW);
                }
                // Velocity effect section
                if (Velocity)
                {
                    pulse = pulse / 4;
                    dXangle += pulse; // dXangle
                    dYangle += pulse; // dYangle
                    dZangle += pulse; // dZangle
                }

                // Very important we must reassign glRC to the new glDC
                // Note: don't use permanent DC, this produce better and smoother display
                GL.wglMakeCurrent(glDC, glRC);

                GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
                GL.glLoadIdentity();
                GL.gluLookAt(0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
                GL.glRotated(dXangle, 1.0, 0.0, 0.0);
                GL.glRotated(dYangle, 0.0, 1.0, 0.0);
                GL.glRotated(dZangle, 0.0, 0.0, 1.0);

                //Draw Diamond
                int RGBColor = LevelColr(nLValue);
                byte R2 = Api.RGBRed(RGBColor);
                byte G2 = Api.RGBGreen(RGBColor);
                byte B2 = Api.RGBBlue(RGBColor);

                RGBColor = LevelColr(nRValue);
                byte R1 = Api.RGBRed(RGBColor);
                byte G1 = Api.RGBGreen(RGBColor);
                byte B1 = Api.RGBRed(RGBColor);

                //Up
                GL.glBegin(GL.GL_TRIANGLE_FAN);
                GL.glColor3ub(R2, G2, B2); GL.glVertex3d(0.0, 1.414, 0.0);
                GL.glColor3ub(R1, G1, B1); GL.glVertex3d(1.0, 0.0, 1.0);
                GL.glColor3ub(0, 0, 0); GL.glVertex3d(1.0, 0.0, -1.0);

                GL.glColor3ub(R1, G1, B1); GL.glVertex3d(-1.0, 0.0, -1.0);
                GL.glColor3ub(0, 0, 0); GL.glVertex3d(-1.0, 0.0, 1.0);
                GL.glColor3ub(R1, G1, B1); GL.glVertex3d(1.0, 0.0, 1.0);
                GL.glEnd();

                //Down
                GL.glBegin(GL.GL_TRIANGLE_FAN);
                GL.glColor3ub(R2, G2, B2); GL.glVertex3d(0.0, -1.414, 0.0);
                GL.glColor3ub(R1, G1, B1); GL.glVertex3d(1.0, 0.0, 1.0);
                GL.glColor3ub(64, 64, 64); GL.glVertex3d(-1.0, 0.0, 1.0);

                GL.glColor3ub(R1, G1, B1); GL.glVertex3d(-1.0, 0.0, -1.0);
                GL.glColor3ub(64, 64, 64); GL.glVertex3d(1.0, 0.0, -1.0);
                GL.glColor3ub(R1, G1, B1); GL.glVertex3d(1.0, 0.0, 1.0);
                GL.glEnd();

                //Refresh display
                GL.wglSwapBuffers(glDC);
                
            }
            Api.ReleaseDC(hGLcontrol, glDC); glDC = 0;
        }

    } // END class MAIN_Form

} // END namespace BlackBox

