﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                              OpenGL class                                |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 10-17-2006 (MM-DD-YYYY)            |
//|                        Last revised : 10-17-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using Win32;

namespace OpenGL
{
    class GL
    {
        private const string OPENGL = "opengl32.dll";
        private const string GLU32 = "glu32.dll";
        private const string USER32 = "user32.dll";
        private const string GDI32 = "GDI32.Dll";

        public const int GL_TRIANGLE_FAN = 0x0006;
        public const int GL_DEPTH_TEST = 0x0B71;
        public const int GL_PROJECTION = 0x1701;
        public const int GL_MODELVIEW = 0x1700;
        public const int GL_DEPTH_BUFFER_BIT = 0x100;
        public const int GL_COLOR_BUFFER_BIT = 0x4000;
        public const int GL_LESS = 0x0201;
        public const int GL_SMOOTH = 0x1D01;
        public const int GL_TEXTURE_2D = 0x0DE1;

        public const uint PFD_TYPE_RGBA = 0;
        public const uint PFD_TYPE_COLORINDEX = 1;

        /* layer types */
        public const uint PFD_MAIN_PLANE = 0;
        public const uint PFD_OVERLAY_PLANE = 1;
        public const uint PFD_UNDERLAY_PLANE = 0xff; // (-1)

        /* PIXELFORMATDESCRIPTOR flags */
        public const uint PFD_DOUBLEBUFFER = 0x00000001;
        public const uint PFD_STEREO = 0x00000002;
        public const uint PFD_DRAW_TO_WINDOW = 0x00000004;
        public const uint PFD_DRAW_TO_BITMAP = 0x00000008;
        public const uint PFD_SUPPORT_GDI = 0x00000010;
        public const uint PFD_SUPPORT_OPENGL = 0x00000020;
        public const uint PFD_GENERIC_FORMAT = 0x00000040;
        public const uint PFD_NEED_PALETTE = 0x00000080;
        public const uint PFD_NEED_SYSTEM_PALETTE = 0x00000100;
        public const uint PFD_SWAP_EXCHANGE = 0x00000200;
        public const uint PFD_SWAP_COPY = 0x00000400;
        public const uint PFD_SWAP_LAYER_BUFFERS = 0x00000800;
        public const uint PFD_GENERIC_ACCELERATED = 0x00001000;
        public const uint PFD_SUPPORT_DIRECTDRAW = 0x00002000;
        public const uint PFD_SUPPORT_COMPOSITION = 0x00008000;

        /* PIXELFORMATDESCRIPTOR flags for use in ChoosePixelFormat only */
        public const uint PFD_DEPTH_DONTCARE = 0x20000000;
        public const uint PFD_DOUBLEBUFFER_DONTCARE = 0x40000000;
        public const uint PFD_STEREO_DONTCARE = 0x80000000;

        [StructLayout(LayoutKind.Sequential)]
        public struct PIXELFORMATDESCRIPTOR
        {
            public ushort nSize;
            public ushort nVersion;
            public uint dwFlags;
            public byte iPixelType;
            public byte cColorBits;
            public byte cRedBits;
            public byte cRedShift;
            public byte cGreenBits;
            public byte cGreenShift;
            public byte cBlueBits;
            public byte cBlueShift;
            public byte cAlphaBits;
            public byte cAlphaShift;
            public byte cAccumBits;
            public byte cAccumRedBits;
            public byte cAccumGreenBits;
            public byte cAccumBlueBits;
            public byte cAccumAlphaBits;
            public byte cDepthBits;
            public byte cStencilBits;
            public byte cAuxBuffers;
            public byte iLayerType;
            public byte bReserved;
            public uint dwLayerMask;
            public uint dwVisibleMask;
            public uint dwDamageMask;
        }

        [DllImport(OPENGL)]
        public static extern bool wglMakeCurrent(int glDC, int glRC);
        [DllImport(OPENGL)]
        public static extern bool wglDeleteContext(int glRC);
        [DllImport(OPENGL)]
        public static extern int wglCreateContext(int glDC);
        [DllImport(OPENGL)]
        public static extern void glClear(uint mask);
        [DllImport(OPENGL)]
        public static extern void glEnable(uint cap);
        [DllImport(OPENGL)]
        public static extern void glShadeModel(uint mode);
        [DllImport(OPENGL)]
        public static extern void glLoadIdentity();
        [DllImport(OPENGL)]
        public static extern void glRotated(double angle, double x, double y, double z);
        [DllImport(OPENGL)]
        public static extern void glBegin(uint mode);
        [DllImport(OPENGL)]
        public static extern void glColor3ub(byte red, byte green, byte blue);
        [DllImport(OPENGL)]
        public static extern void glVertex3d(double x, double y, double z);
        [DllImport(OPENGL)]
        public static extern void glEnd();
        [DllImport(OPENGL)]
        public static extern void glViewport(int x, int y, int width, int height);
        [DllImport(OPENGL)]
        public static extern void glMatrixMode(uint mode);

        [DllImport(GLU32)]
        public static extern void gluLookAt(double eyex, double eyey, double eyez, double centerx, double centery, double centerz, double upx, double upy, double upz);
        [DllImport(GLU32)]
        public static extern void gluPerspective(double fovy, double aspect, double zNear, double zFar);

        [DllImport(GDI32)]
        public static extern int ChoosePixelFormat(int hDC, ref PIXELFORMATDESCRIPTOR ppfd);
        [DllImport(GDI32)]
        public static extern int SetPixelFormat(int hDC, int iPixelFormat, ref PIXELFORMATDESCRIPTOR ppfd);

        [DllImport(OPENGL)]
        public static extern uint wglSwapBuffers(int hdc);

        private static double fovy = 1.25;
        public static double Fovy
        {
            get { return fovy; }
            set { fovy = value; }
        }

        //private static IntPtr parent = IntPtr.Zero;
        //public static IntPtr Parent
        //{
        //    get { return parent; }
        //    set { parent = value; }
        //}

        public static bool SetupPixelFormat(int useDC)
        {
            bool nRet = true;
            PIXELFORMATDESCRIPTOR pfd = new PIXELFORMATDESCRIPTOR();

            //pfd.nSize = 40; // (ushort)sizeof(PIXELFORMATDESCRIPTOR);
            pfd.nSize = (ushort) Marshal.SizeOf(pfd);
            pfd.nVersion = 1;
            pfd.dwFlags = (uint)(PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER); // | PFD_SUPPORT_COMPOSITION);
            pfd.dwLayerMask = PFD_MAIN_PLANE;
            pfd.iPixelType = (byte)PFD_TYPE_RGBA;
            pfd.cColorBits = 24; //32
            pfd.cDepthBits = 16; //16

            pfd.iLayerType = (byte)PFD_MAIN_PLANE;

            int iPixelformat = ChoosePixelFormat(useDC, ref pfd);
            if (iPixelformat != 0)
            {
                if (SetPixelFormat(useDC, iPixelformat, ref pfd) == 0) nRet = false;
            }
            else
            {
                nRet = false;
            }
            return nRet;
        }

    } // END OpenGL interface
}
